<?php

namespace Scheduler\Logic;
//require_once __DIR__.'../../../../../projects/scheduler/public/bootstrap.php';
use \Setforms2\Runtime\Helper\Procedure;
use \Setforms2\Runtime\Helper\Procedure\Param;

class ProcedureRunner{

    protected static $NRORG;

    public static function getNrorg(){
        return self::$NRORG;
    }

    /**
     * Executa uma procedure arbitrária com parâmetros especificados.
     *
     * @param $parametros
     *   Os parâmetros da função, em forma de um json submetido à htmlspecialchars.
     *   Ele deve ter um atributo PROCEDURE_NAME, que é o nome da procedure, e um número qualquer de atributos
     *   correspondentes aos parâmetros da procedure, num formato nome=>valor.
     *
     * @param int $nrorg
     *   O NRORG que será usado em consultas ao banco, etc.
     *
     * @throws \Exception
     */
    public static function executa_procedure($parametros,$nrorg=1){
        //decodifica a string de parâmetros num array
        $array_parametros_recebidos=\Scheduler\Util\ConsumerFunctions::decodeEscapedJson($parametros);
        self::$NRORG=$nrorg; //guarda NRORG para poder ser acessada de outros contextos, caso precise

        $nome=$array_parametros_recebidos['PROCEDURE_NAME'];   //um item do array é o nome
        //os outros são parâmetros (de entrada), e serâo iterados. Portanto é necessário tirar o nome do array
        unset($array_parametros_recebidos['PROCEDURE_NAME']);

        //pega todos pos parâmetros que a procedure têm
        //a procedure precisa de todos os parâmetros sem default definidos, inclusive os de out
        $getProcedureParams="SELECT PARAMETER_NAME, PARAMETER_INOUT, PARAMETER_TYPE FROM TKU_PROCEDURE_PARAM WHERE
PROCEDURE_NAME=:P_PROCEDURE_NAME AND (NRORG=1 OR NRORG=:P_NRORG) AND PARAMETER_INOUT <> 'SEM_PARAMETRO' ORDER BY PARAMETER_POSITION ASC";
        $db = \Setforms2\Runtime\Connection::getInstance();
        $queryParams['P_NRORG']=self::$NRORG;
        $queryParams["P_PROCEDURE_NAME"]=$nome;
        $procedureParams=$db->fetchAll($getProcedureParams, $queryParams);

        //a partir daqui será construída uma string que será executada no banco
        //os parâmetros de out precisam de uma variável atribuida a eles para receberem seu valor.
        //que serâo declaradas aqui.
        $declareString="DECLARE ";
        //essa é a chamada da procedure propriamente dita
        $procedureStmt="BEGIN 
						execute immediate 'alter session set NLS_DATE_FORMAT=''dd/mm/yyyy''';
						$nome(";

        //para cada um dos parâmetros esperados pela procedure
        foreach($procedureParams as $param){
            //pega o valor atribuido para o param da procedure dos params desse script(confuso, sim), se existir
            isset($array_parametros_recebidos[$param["PARAMETER_NAME"]])?
                $valorParametro=$array_parametros_recebidos[$param["PARAMETER_NAME"]]:
                $valorParametro=null;
            //se for do tipo out, cria uma variável dummy pra receber , além de colocar a chamada
            if($param["PARAMETER_INOUT"]==="O"){
                $declareString.=$param["PARAMETER_NAME"];
                ($param["PARAMETER_TYPE"]==2) ? $declareString.=" VARCHAR2(1000);" : $declareString.=" INT;";
                $procedureStmt.=$param["PARAMETER_NAME"]."=>".$param["PARAMETER_NAME"].", ";
            }
            //se for só in, se passa o valor (params IN sem valor são ignorados, assume-se que têm valor default na procedure)
            else if (($valorParametro!==null)){
                $procedureStmt.=$param["PARAMETER_NAME"]."=>".$valorParametro.", ";
            }
        }
        $procedureStmt=rtrim($procedureStmt,", ");    //sobra um ", " no fim da string que deve ser eliminado
        $procedureStmt.="); END;";                    // concatena o END no fim do que deve ser executado
        $execute=$declareString.$procedureStmt;       // junta o declare das variáveis de out e a execução da procedure
		//file_put_contents('c:/temp/log.txt',print_r($execute,true),FILE_APPEND);
        //ao fim, $execute fica assim
        //DECLARE <variaveis> BEGIN <nome da procedure>(<parâmetros de in e out> => <valor ou variável>)
        try{
            $ret = $db->execute($execute);
//            $message="A procedure ".$array_parametros_recebidos['PROCEDURE_NAME']."foi executada.<br/>\n";
//            \Scheduler\Logic\MailSender::sendNotificationEmail($message, $nrorg, $array_parametros_recebidos["RECIPIENT"]);
        }
        catch(\Exception $e){
            $message="Não foi possível rodar a procedure: ".$array_parametros_recebidos['PROCEDURE_NAME']."<br/>\n".$e->getMessage();
            \Scheduler\Logic\MailSender::sendErrorEmail($message, $nrorg, $array_parametros_recebidos["RECIPIENT"]);
            throw new \Exception($message);
        }
    }
}


//    public static function executa_procedure_old($parametros,$nrorg=1){
//
//        $params_decoded=htmlspecialchars_decode($parametros, ENT_QUOTES);
//        $array_parametros_recebidos=json_decode($params_decoded, true);
//        self::$NRORG=$nrorg;
//
//        $nome=$array_parametros_recebidos['PROCEDURE_NAME'];
//        unset($array_parametros_recebidos['PROCEDURE_NAME']);
//
//        $getProcedureParams="SELECT * FROM TKU_PROCEDURE_PARAM WHERE PROCEDURE_NAME=:P_PROCEDURE_NAME AND
//        (NRORG=1 OR NRORG=:P_NRORG) ORDER BY PARAMETER_POSITION ASC";
//
//        $db = \Setforms2\Runtime\Connection::getInstance();
//        $queryParams['P_NRORG']=self::$NRORG;
//        $queryParams["P_PROCEDURE_NAME"]=$nome;
//        $procedureParams=$db->fetchAll($getProcedureParams, $queryParams);
////        $procedureParams=\Scheduler\Entities\TkuProcedureParam::findByProcedureName($nome);
//        $procedure = new Procedure($nome);
//        foreach($procedureParams as $param){
//            $valorParametro=$array_parametros_recebidos[$param["PARAMETER_NAME"]];
//            if (($valorParametro!==null)||($param["PARAMETER_INOUT"]==="O")){
//                $procedure->addParam(new Param($param["PARAMETER_NAME"],$param["PARAMETER_INOUT"],$valorParametro,
//                    $param["PARAMETER_TYPE"],$param["PARAMETER_LENGTH"]));
////                file_put_contents('c:temp/log.txt',print_r($procedure,true));
//            }
//        }
////        foreach($array_parametros_recebidos as $param_name=>$param_value){
////            $procedure->addParam(new Param($param_name,Param::PARAM_INPUT,$param_value,Param::PARAM_TYPE_STR,20));
////        }
//        try{
//            $procedure->call();
////            file_put_contents('c:temp/log.txt',print_r($x,true));
////            file_put_contents('c:temp/log.txt',print_r($procedure,true));
//        }
//        catch(\Exception $e){
//            $message="Não foi possível rodar a procedure: ".$array_parametros_recebidos['PROCEDURE_NAME']."<br/>\n".$e->getMessage();
//            \Scheduler\Logic\MailSender::sendErrorEmail($message, $nrorg, $array_parametros_recebidos["RECIPIENT"]);
//            throw new \Exception($message);
//        }
//
////          throw new Exception("Procedure - Teste de Erro!");
//    }




//ProcedureRunner::executa_procedure("{&quot;P_CONTADOR&quot;:&quot;'SITUTRIB'&quot;,&quot;PROCEDURE_NAME&quot;:&quot;NOVO_CODIGO&quot;}",1);