/* Tecnisa - Tecnologia em Informtica

   Nome: Gera_Trab_Prev - 1070_FINSTPO_GERA_TRAB_PREV.SQL

   Descrio:
   Grava os dados referentes ao relatrio de Fluxo de Caixa (Formulrio)

   Criao:
   02/07/2001 - Andr Rodrigo

   @C:\web_560\sql\end_conv\oracle\1070_FINSTPO_GERA_TRAB_PREV.SQL
*/

CREATE OR REPLACE PROCEDURE GERA_TRAB_PREV
   (P_CDFILIAL    IN TRAB_PREV.CDFILIPREV%TYPE,
    P_NRPREVISAO  IN TRAB_PREV.NRPREVISAO%TYPE,
    P_DTPERIINIC  IN DATE,
    P_DTPERIFINA  IN DATE,
    P_CDEMPRESA   IN VARCHAR2,
    P_CCORR       IN VARCHAR2,
    P_TCCORR      IN VARCHAR2,
    P_SALDOINI    IN VARCHAR2 DEFAULT '1')

AS
   V_NRPAGINA    NUMBER;
   V_NRREGIENCA  NUMBER;
   V_NRCOLUNA    NUMBER;
   V_TOTALDIA    NUMBER;
   V_CTSALDO     NUMBER;
   V_CTCONTA     NUMBER;
   V_TOTMOVIANT  NUMBER;
   V_IDATRASO    VARCHAR2(1);
   V_ISEMPTY     VARCHAR2(1);

   CURSOR C_PERIODO IS
      SELECT DTDATA
      FROM DATA
      WHERE
         DTDATA BETWEEN (P_DTPERIINIC - 1) AND P_DTPERIFINA;

   CURSOR C_BANCOS IS
      SELECT DISTINCT CC.CDEMPRESA, BC.CDBANCO, BC.NMBANCO
        FROM CONTCORR CC, BANCO BC
       WHERE (CC.CDBANCO  = BC.CDBANCO)
         AND (P_CDEMPRESA = 'T' OR INSTR(P_CDEMPRESA, CDEMPRESA) <> 0)
         AND (P_TCCORR    = 'T' OR INSTR(P_CCORR  , CC.CDREDUCONT) <> 0)
       ORDER BY CC.CDEMPRESA, BC.CDBANCO;

   CURSOR C_EMPRESA IS
      SELECT CDEMPRESA, NMEMPRESA
      FROM EMPRESA
      WHERE
         (P_CDEMPRESA = 'T' OR INSTR(P_CDEMPRESA, CDEMPRESA) <> 0)
      ORDER BY CDEMPRESA;

   CURSOR C_CLASSFINANC IS
      SELECT CDCLASSFINA, DSCLASSFINA
      FROM CLASSFINANC
      ORDER BY CDCLASSFINA;

BEGIN
   /* ----- GRAVA O SALDO DOS BANCOS ----- */
   V_NRREGIENCA := 1;
   FOR C_BC IN C_BANCOS LOOP
      V_NRCOLUNA   := 1;
      V_NRPAGINA   := 1;
      FOR C_DATA IN C_PERIODO LOOP
         SELECT COUNT(*) INTO V_CTSALDO
           FROM CONTCORR CC, SDOCCORR SC
          WHERE (CC.CDREDUCONT = SC.CDREDUCONT)
            AND (CC.CDEMPRESA  = C_BC.CDEMPRESA)
            AND (CC.CDBANCO    = C_BC.CDBANCO)
            AND (P_TCCORR      = 'T' OR INSTR(P_CCORR  , CC.CDREDUCONT) <> 0)
            AND (SC.DTSDOCCORR IN (SELECT MAX(DTSDOCCORR)
                                     FROM SDOCCORR
                                    WHERE CDREDUCONT = SC.CDREDUCONT
                                      AND DTSDOCCORR <= C_DATA.DTDATA));

         V_TOTALDIA := 0;
         IF V_CTSALDO > 0 THEN
            SELECT DECODE(P_SALDOINI,'1',SUM(VRSDOCCORR),(SUM(VRSDOCCORR) + SUM(NVL(VRLIMITCREDCCOR,0)))) INTO V_TOTALDIA
              FROM CONTCORR CC, SDOCCORR SC
             WHERE (CC.CDREDUCONT = SC.CDREDUCONT)
               AND (CC.CDEMPRESA  = C_BC.CDEMPRESA)
               AND (CC.CDBANCO    = C_BC.CDBANCO)
               AND (P_TCCORR      = 'T' OR INSTR(P_CCORR  , CC.CDREDUCONT) <> 0)
               AND (SC.DTSDOCCORR IN (SELECT MAX(DTSDOCCORR)
                                        FROM SDOCCORR
                                       WHERE CDREDUCONT = SC.CDREDUCONT
                                         AND DTSDOCCORR <= C_DATA.DTDATA));
         END IF;

         IF V_NRCOLUNA > 1 THEN
            ALTERA_TRAB_PREV(P_CDFILIAL, P_NRPREVISAO, LPAD(TO_CHAR(V_NRREGIENCA), 10, '0'),
                             V_NRCOLUNA, C_DATA.DTDATA, V_TOTALDIA);
         ELSE
            INSERT INTO TRAB_PREV(
               CDFILIPREV,   NRPREVISAO, NRREGIPREV,
               NRPAGIPREV,   CDGRUPPREV, CDEMPRPREV,
               CDITEMPREV,   DSITEMPREV, VRATRASOPREV,
               DTDIA01PREV,  VRDIA01PREV,
               DTINIPERPREV, DTFINPERPREV)
            VALUES(
               P_CDFILIAL,    P_NRPREVISAO, LPAD(TO_CHAR(V_NRREGIENCA), 10, '0'),
               V_NRPAGINA,    '01',         C_BC.CDEMPRESA,
               C_BC.CDBANCO,  C_BC.NMBANCO, 0,
               C_DATA.DTDATA, V_TOTALDIA,
               P_DTPERIINIC,  P_DTPERIFINA);
         END IF;

         /* CONTROLA A COLUNA E A PGINA DO RELATRIO*/
         V_NRCOLUNA   := V_NRCOLUNA + 1;
         IF V_NRCOLUNA > 6 THEN
            V_NRPAGINA   := V_NRPAGINA + 1;
            V_NRCOLUNA   := 1;
            V_NRREGIENCA := V_NRREGIENCA + 1;
         END IF;
      END LOOP;
      V_NRREGIENCA := V_NRREGIENCA + 1;
   END LOOP;

   /* ----- GRAVA O VALOR DAS CONTAS A RECEBER ----- */
   FOR C_EMP IN C_EMPRESA LOOP
      FOR C_CLASS IN C_CLASSFINANC LOOP
         V_NRPAGINA   := 1;

         /* GRAVA OS VALORES DAS CONTAS A RECEBER EM ATRASO */
         SELECT SUM(VRFINALBREC) INTO V_TOTALDIA
         FROM
            (SELECT SUM(CR.VRATUAREC) VRFINALBREC
             FROM
                NOTAFISCAL   NF, CONTARECEBER CR, BOLETRECEBER BO
             WHERE
                BO.CDFILIAL     = CR.CDFILIBOLREC      AND
                BO.NRLANBOLREC  = CR.NRLANBOLREC       AND
                CR.CDEMPRESA    = C_EMP.CDEMPRESA      AND
                CR.CDCLASSFINA  = C_CLASS.CDCLASSFINA  AND
                CR.DTATUAVENREC <= P_DTPERIINIC - 2    AND
                CR.IDSTATREC IN ('A', 'E', 'P')        AND
                NF.CDFILIAL     = CR.CDFILIAL          AND
                NF.NRLANCTONF   = CR.NRLANCTONF        AND
                NF.IDSTATUS  IN ('A', 'E')) CR;

         V_IDATRASO := 'N';
         IF V_TOTALDIA > 0 THEN
            INSERT INTO TRAB_PREV(
               CDFILIPREV,   NRPREVISAO, NRREGIPREV,
               NRPAGIPREV,   CDGRUPPREV, CDEMPRPREV,
               CDITEMPREV,   DSITEMPREV, VRATRASOPREV,
               DTINIPERPREV, DTFINPERPREV)
            VALUES(
               P_CDFILIAL,          P_NRPREVISAO,        LPAD(TO_CHAR(V_NRREGIENCA), 10, '0'),
               V_NRPAGINA,          '02',                C_EMP.CDEMPRESA,
               C_CLASS.CDCLASSFINA, C_CLASS.DSCLASSFINA, V_TOTALDIA,
               P_DTPERIINIC,        P_DTPERIFINA);

               V_IDATRASO := 'S';
         END IF;

         SELECT COUNT(*) INTO V_CTCONTA
         FROM CONTARECEBER
         WHERE
            CDEMPRESA    = C_EMP.CDEMPRESA     AND
            CDCLASSFINA  = C_CLASS.CDCLASSFINA AND
            DTATUAVENREC BETWEEN (P_DTPERIINIC - 1) AND P_DTPERIFINA;

         IF (V_IDATRASO = 'S') OR (V_CTCONTA > 0) THEN
            /* GRAVA OS VALORES DAS CONTAS A RECEBER */
            V_NRCOLUNA := 1;
            FOR C_DATA IN C_PERIODO LOOP

               V_TOTALDIA := 0;
               FOR C_CNTREC IN (
                  SELECT SUM(CR.VRATUAREC) VRFINALBREC
                  FROM
                     NOTAFISCAL   NF,
                     CONTARECEBER CR,
                     BOLETRECEBER BO
                  WHERE
                     BO.CDFILIAL     = CR.CDFILIBOLREC      AND
                     BO.NRLANBOLREC  = CR.NRLANBOLREC       AND
                     CR.CDEMPRESA    = C_EMP.CDEMPRESA      AND
                     CR.CDCLASSFINA  = C_CLASS.CDCLASSFINA  AND
                     CR.DTATUAVENREC = C_DATA.DTDATA        AND
                     CR.IDSTATREC IN ('A', 'E', 'P')             AND
                     NF.CDFILIAL     = CR.CDFILIAL          AND
                     NF.NRLANCTONF   = CR.NRLANCTONF        AND
                     NF.IDSTATUS  IN ('A', 'E')) LOOP
                  IF C_CNTREC.VRFINALBREC IS NULL THEN
                     V_TOTALDIA := V_TOTALDIA + 0;
                  ELSE
                     V_TOTALDIA := V_TOTALDIA + C_CNTREC.VRFINALBREC;
                  END IF;
               END LOOP;

               IF (V_NRCOLUNA > 1) OR (V_IDATRASO = 'S') THEN
                  ALTERA_TRAB_PREV(P_CDFILIAL, P_NRPREVISAO,  LPAD(TO_CHAR(V_NRREGIENCA), 10, '0'),
                                   V_NRCOLUNA, C_DATA.DTDATA, V_TOTALDIA);
               ELSE
                  INSERT INTO TRAB_PREV(
                     CDFILIPREV,   NRPREVISAO, NRREGIPREV,
                     NRPAGIPREV,   CDGRUPPREV, CDEMPRPREV,
                     CDITEMPREV,   DSITEMPREV,
                     DTDIA01PREV,  VRDIA01PREV,
                     DTINIPERPREV, DTFINPERPREV)
                  VALUES(
                     P_CDFILIAL,          P_NRPREVISAO, LPAD(TO_CHAR(V_NRREGIENCA), 10, '0'),
                     V_NRPAGINA,          '02',         C_EMP.CDEMPRESA,
                     C_CLASS.CDCLASSFINA, C_CLASS.DSCLASSFINA,
                     C_DATA.DTDATA,       V_TOTALDIA,
                     P_DTPERIINIC,        P_DTPERIFINA);
               END IF;

               /* CONTROLA A COLUNA E A PGINA DO RELATRIO */
               V_NRCOLUNA   := V_NRCOLUNA + 1;
               IF V_NRCOLUNA > 6 THEN
                  V_NRPAGINA   := V_NRPAGINA + 1;
                  V_NRCOLUNA   := 1;
                  V_NRREGIENCA := V_NRREGIENCA + 1;
                  V_IDATRASO   := 'N';
               END IF;
            END LOOP;
            V_NRREGIENCA := V_NRREGIENCA + 1;
         END IF;
      END LOOP;
   END LOOP;

   /* ----- GRAVA O VALOR DAS CONTAS A PAGAR ----- */
   FOR C_EMP IN C_EMPRESA LOOP
      FOR C_CLASS IN C_CLASSFINANC LOOP
         V_NRPAGINA   := 1;

         /* GRAVA OS VALORES DAS CONTAS A PAGAR EM ATRASO */
         SELECT SUM(VRFINALBPAG) INTO V_TOTALDIA
         FROM
            (SELECT SUM(CP.VRATUAPAG) VRFINALBPAG
             FROM
                NOTAFISCAL NF,
                CONTAPAGAR CP,
                BOLETPAGAR BO
             WHERE
                BO.CDFILIAL     = CP.CDFILIBPAG        AND
                BO.NRLANBOLPAG  = CP.NRLANBOLPAG       AND
                CP.CDEMPRESA    = C_EMP.CDEMPRESA      AND
                CP.DTATUAVENPAG <= P_DTPERIINIC - 2    AND
                CP.CDCLASSFINA  = C_CLASS.CDCLASSFINA  AND
                CP.IDSTATPAG    IN ('A','P')           AND
                NF.CDFILIAL     = CP.CDFILIAL          AND
                NF.NRLANCTONF   = CP.NRLANCTONF        AND
                NF.IDSTATUS     = 'A') CP;

         V_IDATRASO := 'N';
         IF V_TOTALDIA > 0 THEN
            INSERT INTO TRAB_PREV(
               CDFILIPREV,   NRPREVISAO, NRREGIPREV,
               NRPAGIPREV,   CDGRUPPREV, CDEMPRPREV,
               CDITEMPREV,   DSITEMPREV, VRATRASOPREV,
               DTINIPERPREV, DTFINPERPREV)
            VALUES(
               P_CDFILIAL,          P_NRPREVISAO,        LPAD(TO_CHAR(V_NRREGIENCA), 10, '0'),
               V_NRPAGINA,          '03',                C_EMP.CDEMPRESA,
               C_CLASS.CDCLASSFINA, C_CLASS.DSCLASSFINA, -V_TOTALDIA,
               P_DTPERIINIC,        P_DTPERIFINA);

               V_IDATRASO := 'S';
         END IF;

         SELECT COUNT(*) INTO V_CTCONTA
         FROM CONTAPAGAR
         WHERE
            CDEMPRESA    = C_EMP.CDEMPRESA     AND
            CDCLASSFINA  = C_CLASS.CDCLASSFINA AND
            DTATUAVENPAG BETWEEN (P_DTPERIINIC - 1) AND P_DTPERIFINA;

         IF (V_IDATRASO = 'S') OR (V_CTCONTA > 0) THEN
            /* GRAVA OS VALORES DAS CONTAS A PAGAR */
            V_NRCOLUNA := 1;
            FOR C_DATA IN C_PERIODO LOOP

               V_TOTALDIA := 0;
               FOR C_CNTREC IN (
                  SELECT SUM(CP.VRATUAPAG) VRFINALBPAG
                  FROM
                     NOTAFISCAL NF,
                     CONTAPAGAR CP,
                     BOLETPAGAR BO
                  WHERE
                     BO.CDFILIAL     = CP.CDFILIBPAG        AND
                     BO.NRLANBOLPAG  = CP.NRLANBOLPAG       AND
                     CP.CDEMPRESA    = C_EMP.CDEMPRESA      AND
                     CP.DTATUAVENPAG = C_DATA.DTDATA        AND
                     CP.CDCLASSFINA  = C_CLASS.CDCLASSFINA  AND
                     CP.IDSTATPAG    IN ('A','P')           AND
                     NF.CDFILIAL     = CP.CDFILIAL          AND
                     NF.NRLANCTONF   = CP.NRLANCTONF        AND
                     NF.IDSTATUS     = 'A') LOOP
                  IF C_CNTREC.VRFINALBPAG IS NULL THEN
                     V_TOTALDIA := V_TOTALDIA + 0;
                  ELSE
                     V_TOTALDIA := V_TOTALDIA - C_CNTREC.VRFINALBPAG;
                  END IF;
               END LOOP;

               IF (V_NRCOLUNA > 1) OR (V_IDATRASO = 'S') THEN
                  ALTERA_TRAB_PREV(P_CDFILIAL, P_NRPREVISAO,  LPAD(TO_CHAR(V_NRREGIENCA), 10, '0'),
                                   V_NRCOLUNA, C_DATA.DTDATA, V_TOTALDIA);
               ELSE
                  INSERT INTO TRAB_PREV(
                     CDFILIPREV,   NRPREVISAO, NRREGIPREV,
                     NRPAGIPREV,   CDGRUPPREV, CDEMPRPREV,
                     CDITEMPREV,   DSITEMPREV,
                     DTDIA01PREV,  VRDIA01PREV,
                     DTINIPERPREV, DTFINPERPREV)
                  VALUES(
                     P_CDFILIAL,          P_NRPREVISAO, LPAD(TO_CHAR(V_NRREGIENCA), 10, '0'),
                     V_NRPAGINA,          '03',         C_EMP.CDEMPRESA,
                     C_CLASS.CDCLASSFINA, C_CLASS.DSCLASSFINA,
                     C_DATA.DTDATA,       V_TOTALDIA,
                     P_DTPERIINIC,        P_DTPERIFINA);
               END IF;

               /* CONTROLA A COLUNA E A PGINA DO RELATRIO */
               V_NRCOLUNA   := V_NRCOLUNA + 1;
               IF V_NRCOLUNA > 6 THEN
                  V_NRPAGINA   := V_NRPAGINA + 1;
                  V_NRCOLUNA   := 1;
                  V_NRREGIENCA := V_NRREGIENCA + 1;
                  V_IDATRASO   := 'N';
               END IF;
            END LOOP;
            V_NRREGIENCA := V_NRREGIENCA + 1;
         END IF;
      END LOOP;
   END LOOP;

   /* GRAVA OS VALORES DAS PREVISES DE ENTRADA */
   FOR C_EMP IN C_EMPRESA LOOP
      V_NRPAGINA   := 1;
      V_NRCOLUNA   := 1;
      FOR C_DATA IN C_PERIODO LOOP

         V_TOTALDIA := 0;
         FOR C_PREVENT IN
           (SELECT CDCLIENTE, SUM(VRPREVENTR) VRPREVENTR
            FROM PREVENTRADA
            WHERE
               DTPREVENTR = C_DATA.DTDATA AND
               CDEMPRESA  = C_EMP.CDEMPRESA
            GROUP BY CDCLIENTE) LOOP

            SELECT COUNT(CR.CDFILIAL)
            INTO V_CTCONTA
            FROM
               NOTAFISCAL   NF,
               CONTARECEBER CR
            WHERE
               CR.CDEMPRESA     = C_EMP.CDEMPRESA     AND
               CR.DTORIGVENREC  = C_DATA.DTDATA       AND
               NF.CDFILIAL      = CR.CDFILIAL         AND
               NF.NRLANCTONF    = CR.NRLANCTONF       AND
               NF.CDCLIENTE     = C_PREVENT.CDCLIENTE AND
               NF.IDSTATUS  IN ('A', 'E');

            IF V_CTCONTA = 0 THEN
               V_TOTALDIA := V_TOTALDIA + C_PREVENT.VRPREVENTR;
            END IF;
         END LOOP;

         IF (V_NRCOLUNA > 1) THEN
            ALTERA_TRAB_PREV(P_CDFILIAL, P_NRPREVISAO,  LPAD(TO_CHAR(V_NRREGIENCA), 10, '0'),
                             V_NRCOLUNA, C_DATA.DTDATA, V_TOTALDIA);
         ELSE
            INSERT INTO TRAB_PREV(
               CDFILIPREV,   NRPREVISAO, NRREGIPREV,
               NRPAGIPREV,   CDGRUPPREV, CDEMPRPREV,
               CDITEMPREV,   DSITEMPREV,
               DTDIA01PREV,  VRDIA01PREV,
               DTINIPERPREV, DTFINPERPREV)
            VALUES(
               P_CDFILIAL,    P_NRPREVISAO, LPAD(TO_CHAR(V_NRREGIENCA), 10, '0'),
               V_NRPAGINA,    '04',         C_EMP.CDEMPRESA,
               '0000000000',  'E',
               C_DATA.DTDATA, V_TOTALDIA,
               P_DTPERIINIC,  P_DTPERIFINA);
         END IF;

         /* CONTROLA A COLUNA E A PGINA DO RELATRIO */
         V_NRCOLUNA   := V_NRCOLUNA + 1;
         IF V_NRCOLUNA > 6 THEN
            V_NRPAGINA   := V_NRPAGINA + 1;
            V_NRCOLUNA   := 1;
            V_NRREGIENCA := V_NRREGIENCA + 1;
         END IF;
      END LOOP;
      V_NRREGIENCA := V_NRREGIENCA + 1;
   END LOOP;

   /* GRAVA OS VALORES DAS PREVISES DE SAIDA */
   FOR C_EMP IN C_EMPRESA LOOP
      V_NRPAGINA   := 1;
      V_NRCOLUNA   := 1;
      FOR C_DATA IN C_PERIODO LOOP

         SELECT SUM(VRPREVSAIDA)
         INTO V_TOTALDIA
         FROM PREVSAIDA
         WHERE
            DTPREVSAIDA = C_DATA.DTDATA AND
            CDEMPRESA   = C_EMP.CDEMPRESA;

         IF (V_NRCOLUNA > 1) THEN
            ALTERA_TRAB_PREV(P_CDFILIAL, P_NRPREVISAO,  LPAD(TO_CHAR(V_NRREGIENCA), 10, '0'),
                             V_NRCOLUNA, C_DATA.DTDATA, -V_TOTALDIA);
         ELSE

            INSERT INTO TRAB_PREV(
               CDFILIPREV,   NRPREVISAO, NRREGIPREV,
               NRPAGIPREV,   CDGRUPPREV, CDEMPRPREV,
               CDITEMPREV,   DSITEMPREV,
               DTDIA01PREV,  VRDIA01PREV,
               DTINIPERPREV, DTFINPERPREV)
            VALUES(
               P_CDFILIAL,    P_NRPREVISAO, LPAD(TO_CHAR(V_NRREGIENCA), 10, '0'),
               V_NRPAGINA,    '05',         C_EMP.CDEMPRESA,
               '0000000000',  'E',
               C_DATA.DTDATA, -V_TOTALDIA,
               P_DTPERIINIC,  P_DTPERIFINA);
         END IF;

         /* CONTROLA A COLUNA E A PGINA DO RELATRIO */
         V_NRCOLUNA   := V_NRCOLUNA + 1;
         IF V_NRCOLUNA > 6 THEN
            V_NRPAGINA   := V_NRPAGINA + 1;
            V_NRCOLUNA   := 1;
            V_NRREGIENCA := V_NRREGIENCA + 1;
         END IF;
      END LOOP;
      V_NRREGIENCA := V_NRREGIENCA + 1;
   END LOOP;
   
   /* GRAVA OS VALORES DAS CONTAS A RECEBER PREVISTAS */
   FOR C_EMP IN C_EMPRESA LOOP
      FOR C_CLASS IN C_CLASSFINANC LOOP
         V_NRPAGINA   := 1;         
         
         V_TOTALDIA := 0;
         SELECT SUM(CP.VRATUAREC) INTO V_TOTALDIA
         FROM
            (SELECT SUM(CP.VRATUAREC) VRATUAREC
             FROM
                NOTAFISCAL   NF, CONTARECEBER CP
             WHERE
                CP.CDEMPRESA    = C_EMP.CDEMPRESA      AND
                CP.CDCLASSFINA  = C_CLASS.CDCLASSFINA  AND
                CP.DTATUAVENREC <= P_DTPERIINIC - 2    AND
                NF.CDFILIAL     = CP.CDFILIAL          AND
                NF.NRLANCTONF   = CP .NRLANCTONF        AND
                NF.IDSTATUS     = 'V') CP;       
         V_IDATRASO := 'N';
         
         IF V_TOTALDIA > 0 THEN         
            INSERT INTO TRAB_PREV(
               CDFILIPREV,   NRPREVISAO, NRREGIPREV,
               NRPAGIPREV,   CDGRUPPREV, CDEMPRPREV,
               CDITEMPREV,   DSITEMPREV, VRATRASOPREV,
               DTINIPERPREV, DTFINPERPREV)
            VALUES(
               P_CDFILIAL,          P_NRPREVISAO,        LPAD(TO_CHAR(V_NRREGIENCA), 10, '0'),
               V_NRPAGINA,          '06',                C_EMP.CDEMPRESA,
               C_CLASS.CDCLASSFINA, C_CLASS.DSCLASSFINA, -V_TOTALDIA,
               P_DTPERIINIC,        P_DTPERIFINA);               

               V_IDATRASO := 'S';                          
         END IF;        

         SELECT COUNT(*) INTO V_CTCONTA
         FROM
            NOTAFISCAL NF,
            CONTARECEBER CP
         WHERE
            CP.CDEMPRESA    = C_EMP.CDEMPRESA          AND
            CP.CDCLASSFINA  = C_CLASS.CDCLASSFINA      AND
            CP.DTATUAVENREC BETWEEN (P_DTPERIINIC - 1) AND P_DTPERIFINA AND
            NF.CDFILIAL     = CP.CDFILIAL              AND
            NF.NRLANCTONF   = CP.NRLANCTONF            AND
            NF.IDSTATUS     = 'V';

         IF (V_IDATRASO = 'S') OR (V_CTCONTA > 0) THEN
            V_NRCOLUNA   := 1;
            FOR C_DATA IN C_PERIODO LOOP

               SELECT SUM(VRATUAREC)
               INTO V_TOTALDIA
               FROM
                  NOTAFISCAL NF,
                  CONTARECEBER CP
               WHERE
                  CP.CDEMPRESA    = C_EMP.CDEMPRESA     AND
                  CP.CDCLASSFINA  = C_CLASS.CDCLASSFINA AND
                  CP.DTATUAVENREC = C_DATA.DTDATA       AND
                  NF.CDFILIAL     = CP.CDFILIAL         AND
                  NF.NRLANCTONF   = CP.NRLANCTONF       AND
                  NF.IDSTATUS     = 'V';

               IF V_TOTALDIA > 0 THEN
                  V_TOTALDIA := V_TOTALDIA * -1;
               END IF;

               IF (V_NRCOLUNA > 1) OR (V_IDATRASO = 'S') THEN
                  ALTERA_TRAB_PREV(P_CDFILIAL, P_NRPREVISAO,  LPAD(TO_CHAR(V_NRREGIENCA), 10, '0'),
                                   V_NRCOLUNA, C_DATA.DTDATA, V_TOTALDIA);
               ELSE
                  INSERT INTO TRAB_PREV(
                     CDFILIPREV,   NRPREVISAO, NRREGIPREV,
                     NRPAGIPREV,   CDGRUPPREV, CDEMPRPREV,
                     CDITEMPREV,   DSITEMPREV,
                     DTDIA01PREV,  VRDIA01PREV,
                     DTINIPERPREV, DTFINPERPREV)
                  VALUES(
                     P_CDFILIAL,    P_NRPREVISAO, LPAD(TO_CHAR(V_NRREGIENCA), 10, '0'),
                     V_NRPAGINA,    '06',         C_EMP.CDEMPRESA,
                     C_CLASS.CDCLASSFINA,         C_CLASS.DSCLASSFINA,
                     C_DATA.DTDATA, V_TOTALDIA,
                     P_DTPERIINIC,  P_DTPERIFINA);
               END IF;

               /* CONTROLA A COLUNA E A PGINA DO RELATRIO */
               V_NRCOLUNA   := V_NRCOLUNA + 1;
               IF V_NRCOLUNA > 6 THEN
                  V_NRPAGINA   := V_NRPAGINA + 1;
                  V_NRCOLUNA   := 1;
                  V_NRREGIENCA := V_NRREGIENCA + 1;
                  V_IDATRASO   := 'N';
               END IF;
            END LOOP;
            V_NRREGIENCA := V_NRREGIENCA + 1;
         END IF;

      END LOOP;
   END LOOP;


   /* GRAVA OS VALORES DAS CONTAS A PAGAR PREVISTAS */
   FOR C_EMP IN C_EMPRESA LOOP
      FOR C_CLASS IN C_CLASSFINANC LOOP
         V_NRPAGINA   := 1;         
         
         V_TOTALDIA := 0;
         SELECT SUM(CP.VRATUAPAG) INTO V_TOTALDIA
         FROM
            (SELECT SUM(CP.VRATUAPAG) VRATUAPAG
             FROM
                NOTAFISCAL   NF, CONTAPAGAR CP
             WHERE
                CP.CDEMPRESA    = C_EMP.CDEMPRESA      AND
                CP.CDCLASSFINA  = C_CLASS.CDCLASSFINA  AND
                CP.DTATUAVENPAG <= P_DTPERIINIC - 2    AND
                NF.CDFILIAL     = CP.CDFILIAL          AND
                NF.NRLANCTONF   = CP .NRLANCTONF        AND
                NF.IDSTATUS     = 'V') CP;       
         V_IDATRASO := 'N';
         
         IF V_TOTALDIA > 0 THEN         
            INSERT INTO TRAB_PREV(
               CDFILIPREV,   NRPREVISAO, NRREGIPREV,
               NRPAGIPREV,   CDGRUPPREV, CDEMPRPREV,
               CDITEMPREV,   DSITEMPREV, VRATRASOPREV,
               DTINIPERPREV, DTFINPERPREV)
            VALUES(
               P_CDFILIAL,          P_NRPREVISAO,        LPAD(TO_CHAR(V_NRREGIENCA), 10, '0'),
               V_NRPAGINA,          '07',                C_EMP.CDEMPRESA,
               C_CLASS.CDCLASSFINA, C_CLASS.DSCLASSFINA, -V_TOTALDIA,
               P_DTPERIINIC,        P_DTPERIFINA);               

               V_IDATRASO := 'S';                          
         END IF;        

         SELECT COUNT(*) INTO V_CTCONTA
         FROM
            NOTAFISCAL NF,
            CONTAPAGAR CP
         WHERE
            CP.CDEMPRESA    = C_EMP.CDEMPRESA          AND
            CP.CDCLASSFINA  = C_CLASS.CDCLASSFINA      AND
            CP.DTATUAVENPAG BETWEEN (P_DTPERIINIC - 1) AND P_DTPERIFINA AND
            NF.CDFILIAL     = CP.CDFILIAL              AND
            NF.NRLANCTONF   = CP.NRLANCTONF            AND
            NF.IDSTATUS     = 'V';

         IF (V_IDATRASO = 'S') OR (V_CTCONTA > 0) THEN
            V_NRCOLUNA   := 1;
            FOR C_DATA IN C_PERIODO LOOP

               SELECT SUM(VRATUAPAG)
               INTO V_TOTALDIA
               FROM
                  NOTAFISCAL NF,
                  CONTAPAGAR CP
               WHERE
                  CP.CDEMPRESA    = C_EMP.CDEMPRESA     AND
                  CP.CDCLASSFINA  = C_CLASS.CDCLASSFINA AND
                  CP.DTATUAVENPAG = C_DATA.DTDATA       AND
                  NF.CDFILIAL     = CP.CDFILIAL         AND
                  NF.NRLANCTONF   = CP.NRLANCTONF       AND
                  NF.IDSTATUS     = 'V';

               IF V_TOTALDIA > 0 THEN
                  V_TOTALDIA := V_TOTALDIA * -1;
               END IF;

               IF (V_NRCOLUNA > 1) OR (V_IDATRASO = 'S') THEN
                  ALTERA_TRAB_PREV(P_CDFILIAL, P_NRPREVISAO,  LPAD(TO_CHAR(V_NRREGIENCA), 10, '0'),
                                   V_NRCOLUNA, C_DATA.DTDATA, V_TOTALDIA);
               ELSE
                  INSERT INTO TRAB_PREV(
                     CDFILIPREV,   NRPREVISAO, NRREGIPREV,
                     NRPAGIPREV,   CDGRUPPREV, CDEMPRPREV,
                     CDITEMPREV,   DSITEMPREV,
                     DTDIA01PREV,  VRDIA01PREV,
                     DTINIPERPREV, DTFINPERPREV)
                  VALUES(
                     P_CDFILIAL,    P_NRPREVISAO, LPAD(TO_CHAR(V_NRREGIENCA), 10, '0'),
                     V_NRPAGINA,    '07',         C_EMP.CDEMPRESA,
                     C_CLASS.CDCLASSFINA,         C_CLASS.DSCLASSFINA,
                     C_DATA.DTDATA, V_TOTALDIA,
                     P_DTPERIINIC,  P_DTPERIFINA);
               END IF;

               /* CONTROLA A COLUNA E A PGINA DO RELATRIO */
               V_NRCOLUNA   := V_NRCOLUNA + 1;
               IF V_NRCOLUNA > 6 THEN
                  V_NRPAGINA   := V_NRPAGINA + 1;
                  V_NRCOLUNA   := 1;
                  V_NRREGIENCA := V_NRREGIENCA + 1;
                  V_IDATRASO   := 'N';
               END IF;
            END LOOP;
            V_NRREGIENCA := V_NRREGIENCA + 1;
         END IF;

      END LOOP;
   END LOOP;

   V_TOTMOVIANT := 0;
   FOR C_MOVI IN
   (SELECT NRPAGIPREV,
      MAX(DTDIA01PREV) DT01, MAX(DTDIA02PREV) DT02, MAX(DTDIA03PREV) DT03,
      MAX(DTDIA04PREV) DT04, MAX(DTDIA05PREV) DT05, MAX(DTDIA06PREV) DT06,
      SUM(VRATRASOPREV) ATRASO,
      SUM(VRDIA01PREV) DIA01, SUM(VRDIA02PREV) DIA02, SUM(VRDIA03PREV) DIA03,
      SUM(VRDIA04PREV) DIA04, SUM(VRDIA05PREV) DIA05, SUM(VRDIA06PREV) DIA06
    FROM TRAB_PREV
    WHERE
      CDFILIPREV = P_CDFILIAL AND
      NRPREVISAO = P_NRPREVISAO AND
      CDGRUPPREV <> '01'
    GROUP BY NRPAGIPREV) LOOP
      V_NRREGIENCA := V_NRREGIENCA + 1;
      INSERT INTO TRAB_PREV(
         CDFILIPREV,   NRPREVISAO, NRREGIPREV,
         NRPAGIPREV,   CDGRUPPREV, CDEMPRPREV,
         CDITEMPREV,   DSITEMPREV,
         DTDIA01PREV,  VRDIA01PREV,
         DTDIA02PREV,  VRDIA02PREV,
         DTDIA03PREV,  VRDIA03PREV,
         DTDIA04PREV,  VRDIA04PREV,
         DTDIA05PREV,  VRDIA05PREV,
         DTDIA06PREV,  VRDIA06PREV,
         DTINIPERPREV, DTFINPERPREV)
      VALUES(
         P_CDFILIAL,    P_NRPREVISAO, LPAD(TO_CHAR(V_NRREGIENCA), 10, '0'),
         C_MOVI.NRPAGIPREV,    '99',         '',
         '',         '',
         C_MOVI.DT01, V_TOTMOVIANT + NVL(C_MOVI.ATRASO, 0),
         C_MOVI.DT02, V_TOTMOVIANT + C_MOVI.DIA01 + NVL(C_MOVI.ATRASO, 0),
         C_MOVI.DT03, V_TOTMOVIANT + C_MOVI.DIA01 + C_MOVI.DIA02 + NVL(C_MOVI.ATRASO, 0),
         C_MOVI.DT04, V_TOTMOVIANT + C_MOVI.DIA01 + C_MOVI.DIA02 + C_MOVI.DIA03 + NVL(C_MOVI.ATRASO, 0),
         C_MOVI.DT05, V_TOTMOVIANT + C_MOVI.DIA01 + C_MOVI.DIA02 + C_MOVI.DIA03 + C_MOVI.DIA04 + NVL(C_MOVI.ATRASO, 0),
         C_MOVI.DT06, V_TOTMOVIANT + C_MOVI.DIA01 + C_MOVI.DIA02 + C_MOVI.DIA03 + C_MOVI.DIA04 + C_MOVI.DIA05 + NVL(C_MOVI.ATRASO, 0),
         P_DTPERIINIC,  P_DTPERIFINA);

      V_TOTMOVIANT := V_TOTMOVIANT + NVL(C_MOVI.ATRASO, 0) +
                      NVL(C_MOVI.DIA01, 0) + NVL(C_MOVI.DIA02, 0) + NVL(C_MOVI.DIA03, 0) +
                      NVL(C_MOVI.DIA04, 0) + NVL(C_MOVI.DIA05, 0) + NVL(C_MOVI.DIA06, 0);
   END LOOP;
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('GERA_TRAB_PREV','5.60.xxx'); END;
/

